//header for SimpleCfg
//Last changed 30.04.2007

#include"utilities.h" 

//-----class SimpleCfg-------------

//this is the simplified class for a configuration
//i.e. modelling compound particles as atoms with given radius and with a square box
class SimpleCfg
{
	public:
		SimpleCfg(); //default constructor
		SimpleCfg(SimpleCfg &source); //copy constructor

		~SimpleCfg(); //destructor
		static int ntotal;//total number of atoms
		static int ntypes;// number of atom types
		static int npar_types;// number of atom types in parameter file
		static int npartials;//number of partial
		static int nconfig;//number of configurations
		static int option;//type of the configuration file

		static int ntot_used;//total nmber of used atoms for CrystalMaker
		static int choose_part;//option switch to regulate how the particles to be used by CrystalMaker should be chosen
		static int select_axis;//the cutting planes are perpendicular to the selected axis, if choose_part==4 (values can be 0 for x, 1 for y and 2 for z)
		static int *pnused_atoms;//number of atoms used from each type (the first X atom for each type)
		static int *pnatoms;//pointer to number of atoms (per type)
		static int *cumul;//array of cumulative number of atoms per type
				//i.e (0,ntype1, ntype1 + ntype2,...,ntotal-nlasttype,ntotal))
		static int *indices;//indices (in the configuration, between 0->(nparticles-1)) of the particles in increasin order to be selected for CrystalMaker
		static int **nmoves;//pointer to number of generated, tried and accepeted moves
		static int **ind_finder;//beginning of the indices for each type in indices array
		static int *colour_code;//arbitrary colour coding for the particle types
		
		static double boxedge;//half-length of the box edge (in Angstrom)
		static double coord_limit1;//the first limiting value of the selected axis, if choose_part==4
		static double coord_limit2;//the second limiting value of the selected axis, if choose_part==4
		static char (*particle_name)[60];//name of the particles/type. Only the first two charcters will be saved into the CrystalMaker configuration file
		
		double *positions;//pointer to array of (x,y,z) positions, after a new move is made, the NEW positions of the moved atom(s) are kept here!!!
		double **finder;//array of pointers to the start of the coordinates
						//for one atom type in the positions array

		//member functions
		static void SetArraysize(int **apointer, int size);//sets the size of the dynamic array
		static int GetParams_cfg(ifstream &file);//Reading the static parameters (number of atoms, types, timeframes) from the cfg file
												  //and initializing the static arrays pnatoms, cumul
												  //Setting the static parameters for the SimpleCfg object 
													//gives back 1, if it was succesfull, 0 if failed
		static void GetParams_bcf(ifstream &file);//Reading the static parameters (number of atoms, types, timeframes) from the cfg file
												  //and initializing the static arrays pnatoms, cumul
												  //Setting the static parameters for the SimpleCfg object 
													//gives back 1, if it was succesfull, 0 if failed
		static void GetCrystalMakerPar(char *filename, char *incfg, char *outcfg);//reading the parameters rgarding the building of the CrystalMaker configuration file from te parameter file
		//Calculating the indices of the particles to be selected distributed uniformly over the available index range of the given type 
		static void AssignIndex();
		//Calculating the indices of the particles positinoned between the two cutting planes
		void SelectIndex();
		//In case of choose_part=3, check, whether the indices are in range
		void CheckIndex();
		void Copy(SimpleCfg &source);//copy to the target
		void Save(char *filename, char *title) const;//save to a file
		void Save_binary(char *filename) const;//save the SimpleCfg object in binary format
		void SaveCrystalMaker(char *filename) const;//save the CrystalMaker Extended format text file
		void Load(ifstream &file);//loads from a text file
		int Load_binary(ifstream &file);//loading the data from a binary file,
										//gives back 1, if it was succesfull, 0 if failed
		int Comp(const SimpleCfg &conf) const;//compare the coordinates of the caller object with conf
		
		
		
};
	
//destructor of the SimpleCfg class
inline SimpleCfg::~SimpleCfg()
{
	if (positions!=0)
	{
		//to make sure, that the program will not try to delete the array,
		//if it was already deleted 
		delete positions;
		positions=0;
	}
	if (finder!=0)
	{
		//to make sure, that the program will not try to delete the array,
		//if it was already deleted 
		delete finder;
		finder=0;
	}
	cout<<"SimpleCfg destructor"<<endl;
};
