//source SimpleCfg.cpp
//Last changed 06.09.2007
#include "Coordinates.h"

int  SimpleCfg::ntotal;//total number of atoms
int  SimpleCfg::ntypes;// number of atom types
int  SimpleCfg::npar_types;// number of atom types in parameter file
int  SimpleCfg::npartials;//number of partials
int  SimpleCfg::nconfig;//number of configurations
int  SimpleCfg::option;//type of the configuration file
int  SimpleCfg::choose_part;//option switch to regulate how the particles to be used by CrystalMaker should be chosen
int  SimpleCfg::select_axis;//the cutting planes are perpendicular to the selected axis, if choose_part==4
int	 SimpleCfg::ntot_used;//total nmber of used atoms for CrystalMaker
int *SimpleCfg::pnatoms=0;
int *SimpleCfg::pnused_atoms=0;//number of atoms used from each type (the first X atom for each type)
int *SimpleCfg::cumul=0;
int *SimpleCfg::indices;//indices of the atoms to be selected for CrystalMaker
int **SimpleCfg::nmoves;//pointer to number of generated, tried and accepeted moves
int **SimpleCfg::ind_finder;//beginning of the indices for each type in indices array

double SimpleCfg::boxedge;
double SimpleCfg::coord_limit1;//the first limiting value of the selected axis, if choose_part==4
double SimpleCfg::coord_limit2;//the second limiting value of the selected axis, if choose_part==4
char (*SimpleCfg::particle_name)[60];//name of the particles /type

//-----------constructor----------
//BEWARE: objects created by the constructor are meaningless 
//and they should be initialised by loading data from a file	
SimpleCfg::SimpleCfg()//default constructor
{
	int i;
	positions=new double[3*ntotal];
	finder=new double*[ntypes];//pointers to the coord of first atom of a
								// given type in the coordinates array
	if (positions== NULL || pnatoms==NULL)
	{
		cout << "Sorry, cannot create SimpleCfg."<<endl;
		cleanexit();
	}
	//initialising the finder
	for(i=0;i<ntypes;i++)
		*(finder+i)=positions+(3* cumul[i]);
}

SimpleCfg::SimpleCfg(SimpleCfg &source)//copy constructor, source to the new object
{
	//*****WARNING******
	//there must be enough space in the target to
	//store all the source features
	
	int i;
	double *pos1, *pos2;

	positions=new double[3*ntotal];
	finder=new double*[ntypes];//pointers to the coord of first atom of a
								// given type in the coordinates array
	//check
	if(positions==0)
	{
		cout<<"\n****WARNING****"<<endl;
		cout<<"SimpleCfg::Copy - the target SimpleCfg object is not initialised!"\
		<<endl;
		cout<<"Cannot run this way, exiting... "<<endl;
		cleanexit();
	}
	
	pos1=source.positions;
	pos2=positions;
	for(i=0;i<ntotal;i++)
	{
		*pos2++=*pos1++;
		*pos2++=*pos1++;
		*pos2++=*pos1++;
	}

	//initialising the finder
	for(i=0;i<ntypes;i++)
		*(finder+i)=positions+(3* cumul[i]);
}

//- - - - - - - - - - -  
//Reading the static parameters (number of atoms, types) from the cfg file
//and initializing the static arrays pnatoms, cumul
int SimpleCfg::GetParams_cfg(ifstream &file)
{
	int *typ;
	int i;

	if (CheckFileState(file,"SimpleCfg::GetParams_cfg - *.cfg")==0)
		return(0);//the *.cfg could not be opened, probably dos not exist
	
	skipline(file);//comment
	skipline(file);//component
	skipline(file);
	skipline(file);
	skipline(file);//moves generated, tried, accepted
	skipline(file);
	file>>ntotal;//total number of atoms
	skipline(file);
	file>>ntypes;//number of atom types
	if (ntypes!=npar_types)
	{
		cout<<"The number of atom types ("<<npar_types<<") in the parametr file is not the same, as the number of types "<<endl;
		cout<<"in the configuration file ("<<ntypes<<"). Cannot run this way, exiting..."<<endl;
		cleanexit();
	}
	skipline(file);
	skipline(file);
	skipline(file);
	skipline(file);
	skipline(file);//box shape
	skipline(file);//defining vectors are:
	file>>boxedge;//half-length of the box
	skipline(file);
	skipline(file);
	skipline(file);
	skipline(file);

	//Setting the size of the arrays pnatos,cumul
	SetArraysize(&pnatoms,ntypes);
	SetArraysize(&cumul,ntypes+1);
	
	nmoves=new int*[3];
	npartials=ntypes*(ntypes+1)/2;


	//loading the number of atoms for each type
	typ=pnatoms;
	for(i=0;i<ntypes;i++)
	{
		file>>pnatoms[i];
		skipline(file);
		skipline(file);
		skipline(file);
		skipline(file);
		if (pnused_atoms[i]>pnatoms[i])
		{
			cout<<"The number of used atoms for type "<<i+1<<" (type index begin with 1) is "<<pnused_atoms[i]<<", which is greater than"<<endl;
			cout<<"the total number of particles for this type, which is "<<pnatoms[i]<<"! Setting the  number of used particles"<<endl;
			cout<<"to "<<pnatoms[i]<<"."<<endl;
			pnused_atoms[i]=pnatoms[i];
		}
	}

	//initialising the cumul array
	*cumul=0;//first element
	for(i=1;i<=ntypes;i++)
	{
		*(cumul+i)=*(cumul+i-1)+*(pnatoms+i-1);
	}

	//Checking, whether loading was successfull
	CheckReadFileState(file,"SimpleCfg::GetParameters_cfg");
	
	return (1);//it was succesfull
	
}

//---------------------------------------------------------------------
//Reading the static parameters (number of atoms, types) from the bcf file
//and initializing the static arrays pnatoms, cumul
void SimpleCfg::GetParams_bcf(ifstream &file)
{
	int i,k=3;
	

	//check, whether the file is open
	if (CheckFileState(file,"SimpleCfg::GetParams_bcf *.bcf")==0)
	{
		//the *.bcf could not be opened, probably dos not exist
		cout<<"Cannot run this way, exiting"<<endl;
		cleanexit();
	}

	
	//reading the length of the longint variable
	file.read(reinterpret_cast<char *>(&i), sizeof(i));
	if (i==4 || i==8)
	{
		k=i*3/4;
		//number of atoms
		file.read(reinterpret_cast<char *>(&SimpleCfg::ntotal), sizeof(SimpleCfg::ntotal));
	}
	else
		ntotal=i;
		
	
	//number of atom types
	file.read(reinterpret_cast<char *>(&SimpleCfg::ntypes), sizeof(SimpleCfg::ntypes));
	if (ntypes!=npar_types)
	{
		cout<<"The number of atom types ("<<npar_types<<") in the parametr file is not the same, as the number of types "<<endl;
		cout<<"in the configuration file ("<<ntypes<<"). Cannot run this way, exiting..."<<endl;
		cleanexit();
	}
	npartials=ntypes*(ntypes+1)/2;

	//Setting the size of the arrays pnatos,cumul
	SetArraysize(&pnatoms,ntypes);
	SetArraysize(&cumul,ntypes+1);
	nmoves=new int*[3];
	//loading the number of atoms for each type

	//number of atoms/type
	for (i=0;i<ntypes;i++)
	{
		file.read(reinterpret_cast<char *>(&SimpleCfg::pnatoms[i]), (sizeof(*SimpleCfg::pnatoms)));
		if (pnused_atoms[i]>pnatoms[i])
		{
			cout<<"The number of used atoms for type "<<i+1<<" (type index begin with 1) is "<<pnused_atoms[i]<<", which is greater than"<<endl;
			cout<<"the total number of particles for this type, which is "<<pnatoms[i]<<"! Setting the  number of used particles"<<endl;
			cout<<"to "<<pnatoms[i]<<"."<<endl;
			pnused_atoms[i]=pnatoms[i];
		}
		
	}
	
	//half size of the simulation cell
	file.read(reinterpret_cast<char *>(&SimpleCfg::boxedge), sizeof(SimpleCfg::boxedge));

	//initialising the cumul array
	*cumul=0;//first element
	for(i=1;i<=ntypes;i++)
	{
		*(cumul+i)=*(cumul+i-1)+*(pnatoms+i-1);
	}

	//Checking, whether loading was successfull
	CheckReadFileState(file,"SimpleCfg::GetParameters_bcf");

	
}
//-------------Allocating memory for static arrays----------------------
void SimpleCfg::SetArraysize(int **apointer, int size)
{
	*apointer=new int[size];
	return;
}


//---------------Copy source to target---------------	
void SimpleCfg::Copy(SimpleCfg &source) 
{
	//*****WARNING******
	//there must be enough space in the target to
	//store all the source features
	
	//check

	int i;
	double *pos1, *pos2;
	
	pos1=source.positions;
	pos2=positions;
	for(i=0;i<ntotal;i++)
	{
		*pos2++=*pos1++;
		*pos2++=*pos1++;
		*pos2++=*pos1++;
	}
}
	
//---------------Save the configuration created into RMCSANS version1 text file---------------	

void SimpleCfg::Save(char *filename, char *title) const
{
	int i;
	double *pos;
	int *numb;
	int dumbint=0;
	ofstream file;

	OpenFile(file,filename,"SimpleCfg::Save",0);//open file, check, whether it was successfully opened

	file.setf(ios::fixed|ios::right);
	
	file<<" (RMCSANS version 1 format configuration file) !file created by SimpleCfg::save !"<<endl;
	file<<title;
	file<<"\n\n"<<endl;
	file<<" ";
	for (i=0;i<3;i++)
		file<<j10<<*nmoves[i];
	file<<" moves generated, tried, accepted "<<endl;
	file<<" "<<j10<<dumbint<<"                     configurations saved\n "<<endl;
	file<<" "<<j10<<ntotal<<" atoms of all types"<<endl;
	file<<" "<<j10<<ntypes<<" types of atoms "<<endl;
	file<<" "<<j10<<1<<" (to be compatible with RMC)"<<endl;
	file<<" "<<j10<<0<<" (to be compatible with RMC)\n "<<endl;
	file<<" "<<j10<<"F"<<" (box is cubic)	"<<endl;
	file<<"            Defining vectors are: "<<endl;

	file.precision(6);
	file<<" "<<j10<<" "<<" "<<j10<<boxedge<<" "<<j10<<0.0<<" "<<j10<<0.0<<endl;
	file<<" "<<j10<<" "<<" "<<j10<<0.0<<" "<<j10<<boxedge<<" "<<j10<<0.0<<endl;
	file<<" "<<j10<<" "<<" "<<j10<<0.0<<" "<<j10<<0.0<<" "<<j10<<boxedge<<endl;
	file<<endl;

	numb=pnatoms;
	for(i=1;i<=ntypes;i++)
	{
		file<<" "<<j10<<*numb++<<" atoms of type"<<"  "<<i<<endl;
		file<<" "<<j10<<1<<" atom sites "<<endl;
		file<<" "<<j10<<" "<<" "<<j10<<0.00<<" "<<j10<<0.00<<" "<<j10<<0.00<<"\n "<<endl;
	}
		
	file.precision(15);
	pos=positions;
	for(i=0;i<ntotal;i++)
	{
		file<<" "<<j10<<*pos++<<" ";
		file<<" "<<j10<<*pos++<<" ";
		file<<" "<<j10<<*pos++<<endl;
	};
	file.unsetf(ios::fixed|ios::right);
	file.close();

};

//----------------Save the configuration into RMCSANS version1 binary file-----------------------
void SimpleCfg::Save_binary(char *filename) const
{

	long int configsize;
	ofstream file;

	OpenFile(file,filename,"SimpleCfg::Save_binary",1);//open file, check, whether it was successfully opened

	file.write(reinterpret_cast<const char *>(&ntotal), sizeof(ntotal));
	file.write(reinterpret_cast<const char *>(&ntypes), sizeof(ntypes));
		file.write(reinterpret_cast<const char *>(pnatoms), (ntypes*sizeof(*pnatoms)));
	file.write(reinterpret_cast<const char *>(&boxedge), sizeof(boxedge));

	configsize=3*ntotal*sizeof(*positions);
	file.write(reinterpret_cast<const char *>(positions),configsize);
	file.close();

}

//---------------Loads the configuration created in RMCSANS version1---------------	
void SimpleCfg::Load(ifstream &file)
{
	int *typ;
	int i,dumb;
	double *pos;
	
	//the file exist, as it was checked before by GetParams, just in case check it again
	if (CheckFileState(file,"SimpleCfg::GetParams_cfg - *.cfg")==0)
	{
		cout<<"The *.cfg file was found earlier, but now it is not available, cannot continue, exiting..."<<endl;
		cleanexit();//the *.cfg could not be opened, probably dos not exist
	}
	
		
	skipline(file);//comment
	skipline(file);//component
	skipline(file);
	skipline(file);
	skipline(file);//moves generated, tried, accepted
	skipline(file);
	file>>ntotal;//total number of atoms
	skipline(file);
	file>>ntypes;//number of atom types
	skipline(file);
	skipline(file);
	skipline(file);
	skipline(file);
	skipline(file);//box shape
	skipline(file);//defining vectors are:
	file>>dumb;//half-length of the box is not read in case it was reset
	skipline(file);
	skipline(file);
	skipline(file);
	skipline(file);

	//loading the number of atom types for each type
	typ=pnatoms;
	for(i=1;i<=ntypes;i++)
	{
		file>>*typ++;
		skipline(file);
		skipline(file);
		skipline(file);
		skipline(file);
	}

	//now loading the positions	
	pos=positions;
	for(i=0;i<ntotal;i++)
	{
		file>>*pos++;
		file>>*pos++;
		file>>*pos++;
	}

	CheckReadFileState(file,"SimpleCfg::Load");
	
};

//----Loads the configurations written in RMCSANS version1 binary format----
//------------gives back 1, if it was succesfull, 0 if failed---------------
int SimpleCfg::Load_binary(ifstream &file)			
{
	if (file.is_open()==0 || !file)   //verification that file is open
	{
		cout<<"SimpleCfg::Load_binary - file not open or non-existing."<<endl;
		cout<<"The configurations from the text type .cfg file will be used!"<<endl;
		return(0);
	};
	
	int i;
	int temp_i;//data type has to be the same, as the parameters, that is red into it!
	double temp_d;//data type has to be the same as boxedge's
	long int configsize;

	//reading the length of the longint variable
	file.read(reinterpret_cast<char *>(&i), sizeof(i));
	if (i==4 || i==8)
	{
		//number of atoms
		file.read(reinterpret_cast<char *>(&temp_i), sizeof(SimpleCfg::ntotal));
	}
	else
		temp_i=i;
	if (temp_i!=ntotal)
	{
		cout<<"INCONSISTENCY! The number of atoms is "<<ntotal<<" from the .cfg file, and "<<endl;
		cout<<temp_i<<" from the binary .bcf file!"<<endl;
		cout<<"The binary file will be ignored, the data from the .cfg file will be used!"<<endl;
		return(0);
	}
	//number of atom types
	file.read(reinterpret_cast<char *>(&temp_i), sizeof(ntypes));
	if (temp_i!=ntypes)
	{
		cout<<"INCONSISTENCY! The number of atom types is "<<ntypes<<" from the .cfg file, and "<<endl;
		cout<<temp_i<<" from the binary .bcf file!"<<endl;
		cout<<"The binary file will be ignored, the data from the .cfg file will be used!"<<endl;
		return(0);
	}
	//number of atoms/type
	for (i=0;i<ntypes;i++)
	{
		file.read(reinterpret_cast<char *>(&temp_i), (sizeof(*pnatoms)));
		if (temp_i!=pnatoms[i])
		{
			cout<<"INCONSISTENCY! The number of atoms for type "<<i+1<<" is "<<pnatoms[i]<<" from the .cfg file, and "<<endl;
			cout<<temp_i<<" from the binary .bcf file! (Starting with 1)"<<endl;
			cout<<"The binary file will be ignored, the data from the .cfg file will be used!"<<endl;
			return(0);
		}
	}
	//half size of the simulation cell
	file.read(reinterpret_cast<char *>(&temp_d), sizeof(boxedge));//not checked, as itmight have been reset byRunParams::GetParams

	if (file.is_open()==0 || !file)//Checking, whether the load was successful sofar
	{
		cout<<"Loading of the main parameters of the configurations from the .bcf file has failed!"<<endl;
		CheckReadFileState(file,"SimpleCfg::::GetParameters_bcf");
	}
	//Loading of the coordinates
	//the size of the configuration in bytes
	configsize=3*ntotal* sizeof(*positions);
		
	file.read(reinterpret_cast<char *>(positions),configsize);

	if (file.is_open()==0 || !file)//Checking, whether the load was successful
	{
		CheckReadFileState(file,"SimpleCfg::Load_binary");
		return(0);
	}
	return(1);//the load was successful
}

//-------------save the CrystalMaker Extended format text file----------------------------
void SimpleCfg::SaveCrystalMaker(char *filename) const
{
	int i,j,itype;
	int count;
	int *ind;
	double *pos;
	ofstream file;

	OpenFile(file,filename,"SimpleCfg::SaveCrystalMaker",0);//open file, check, whether it was successfully opened
	file.precision(6);//Set floating precision
	file.setf(ios::fixed|ios::right);
  

	int sum=0;//Compute the sum of all particles
	for (i=0;i<ntypes;i++)
		sum += pnused_atoms[i];
  

	file<<"MOLE"<<endl;
	file<<"ATOM"<<endl;
	file.precision(7);
	
	if (choose_part==2)//first pnused_atoms sphers of a type will be saved
	{
		for (itype=0; itype<ntypes; itype++)
		{
		   
			pos=finder[itype];//sets the pointer to the first coordinate of this type
			for (i=0; i<pnused_atoms[itype]; i++)//write the x,y,z coordintes of the first pnused_atom atom of each type  
			{
				file<<particle_name[itype]<<" "<<particle_name[itype]<<" ";//Name of the atom type twice
				for (j=0;j<3;j++)
				{
					file<<(*pos+1.000)*SimpleCfg::boxedge<<" ";
					pos++;
				}
				file<<endl;
			}
		}
	}
	else
	{
	
		ind=indices;//indices of the selected particles
		//Only the atoms with indices in indices array will be selected for output
		for (itype=0; itype<ntypes; itype++)
		{
			pos=finder[itype];//sets the pointer to the first coordinate of this type
			//write the x,y,z coordintes of the selected atoms  
			count=0;
			for (i=cumul[itype]; i<cumul[itype+1]; i++)
			{
				if (*ind==i)//this particle is selected
				{
					file<<particle_name[itype]<<" "<<particle_name[itype]<<" ";//Name of the atom type twice
					for (j=0;j<3;j++)
					{
						file<<(*pos+1.000)*SimpleCfg::boxedge<<" ";
						pos++;
					}
					file<<endl;
					count++;//increase the number of atoms saved
					if (count==ntot_used)
						break;//all the selected atoms were saved, leave cycle
					ind++;
				}
				else
					pos+=3;//this particle was not selected, go to the coordines of the next
			}
		}
	}
	file.unsetf(ios::fixed|ios::floatfield);
	file.close();
};

//----reading the parameters rgarding the building of the CrystalMaker configuration file from te parameter file------
void SimpleCfg::GetCrystalMakerPar(char *filename,char *incfg, char *outcfg)
{
	int i,tot;
	double temp;
	char *ext;
	ext=new char[10];
	char *pchar;
	ifstream file;


	tot=0;


	file.open(filename);
	//the file exist, as it was checked before by GetParams, just in case check it again
	if (CheckFileState(file,"SimpleCfg::GetCrystalMakerPar CrystalMaker parameter")==0)
		cleanexit();//the parameter file could not be opened, probably dos not exist
	
	//The following input and output file will be used based on filename:
	//Text type input:filename.cfg or binary type input: filename.bcf
	//output:filename_CM.cfg 
	file>>incfg;
	skipline(file);
	pchar=strchr(incfg,'.');
	if (strcmp(pchar+1,"cfg")==0)
	{
		option=1;
		if (strncmp(pchar-5,"_coll",4)==0)
			option=3;
	}
	else
	{
		if (strcmp(pchar+1,"bcf")==0)
			option=2;
		else
		{
			cout<<"Unrecognized file extension for input file, has to be either *.cfg, *.cfg_coll or *.bcf!"<<endl;
			cout<<"Cannot run this way, exit..."<<endl;
			cleanexit();
		}
	}
	strncpy(outcfg,incfg,pchar-incfg);
	outcfg[(int)(pchar-incfg)]='\0';

	
	file>>nconfig;
	if (option!=3 && nconfig>1)
	{
		cout<<"In case of a single text type (*.cfg) or a binary (*.bcf) file there can be just one configuration in a file, setting number of configuration to 1!"<<endl;
		nconfig=1;
	}

	
	skipline(file);
	file>>npar_types;
	skipline(file);
	//Setting the arrays
	SetArraysize(&pnused_atoms,npar_types);
	particle_name=new char[npar_types][60];
	ind_finder=new int*[npar_types];

	file>>choose_part;//the option switch for how the particles should be chosen, if not all particles of a type are used
	if (choose_part<1 || choose_part>4)
	{
		cout<<"GetCrystalMakerPar: The value of the choose_part option is outside the acceptable range (1-4)! It will be set to default 1!"<<endl;
		choose_part=1;
	}
	
	if (choose_part==4)
	{
		file>>select_axis;
		if (select_axis<0 || select_axis>2)
		{
			cout<<"The value of the axis perpendicular to the cutting planes can be 0,1 or 2!"<<endl;
			cout<<"Cannot run this way, exiting..."<<endl;
			cleanexit();
		}
		file>>coord_limit1;//limiting values for the coordinates along the selected axis
		file>>coord_limit2;
		if (coord_limit1<-1 || coord_limit1>1 || coord_limit2<-1 || coord_limit2>1)
		{
			cout<<"The limiting values for the reduced coordinates along the selected axis has to be between -1 and +1!"<<endl;
			cout<<"Cannot run this way, exiting..."<<endl;
			cleanexit();
		}
		if (coord_limit1>coord_limit2)//swap the limits, if linmit1>limit2
		{
			temp=coord_limit1;
			coord_limit1=coord_limit2;
			coord_limit2=temp;
		}
	}
	skipline(file);
	for (i=0;i<npar_types;i++)
	{
		file>>particle_name[i];
		if (choose_part<4)
		{		
			file>>pnused_atoms[i];
			ntot_used+=pnused_atoms[i];//total number of used particles
		}
		
		
		CheckReadFileState(file,"Simple_Cfg::GetCrystalMakerPar");//check, whether the reading was OK so far 
		if (i<npar_types-1 || choose_part==3)
			skipline(file);
		
	};

	//create the array for the indices, if necessary
	i=0;//default
	if (choose_part==1 || choose_part==3)
		i=ntot_used;
	indices=new int[i];
	//for choose_part==4 it willl be set later
	
	//Read the indices of the selected atoms, if choose_part==3
	if (choose_part==3)
	{
		for (i=0;i<ntot_used;i++)
		{
			indices[i]=0;//not to cause out of range error, if was not read into it
			file>>indices[i];//indices of the atoms to be selected, absolute indices in the configuration (1->ntotal)
			indices[i]--;// to put it in the range 0->(nparticles-1)
		
		}
		CheckReadFileState(file,"Simple_Cfg::GetCrystalMakerPar");//check, whether the reading was OK so far 
		
		//ordering the particle indices
		O_Sort(ntot_used,indices);
	}

	//Initializing the finder for the indices, required for choose_part=1,3 it will be set later for 4
	if (choose_part==1 || choose_part==3)
	{
		tot=0;
		for (i=0;i<npar_types;i++)
		{
			ind_finder[i]=indices+tot;
			tot+=pnused_atoms[i];
		}
	}




	CheckReadFileState(file,"SimpleCfg::GetCrystalMakerPar");
	file.close();

};

//Calculating the indices of the particles to be selected distributed uniformly over the available index range of the given type 
void SimpleCfg::AssignIndex()
{
	int n,itype;
	int step,remainder,inc_step,count,n_assigned,present_step;
	int last_assigned,last_to_assign;
	ofstream outfile;
outfile.open("index");
	remainder=0;
	

	for (itype=0;itype<ntypes;itype++)
	{
		//calculating the step and the reaminder for the assignment
		if (pnused_atoms[itype]>1)
		{
			step=(int)(pnatoms[itype]-1)/(pnused_atoms[itype]-1);
			last_assigned=step*(pnused_atoms[itype]-1);//this is the last index to be assigned with step starting with 1
			last_to_assign=pnatoms[itype]-1;//this is the index of the last index that should be assigned to this type
			remainder=last_to_assign-last_assigned;//this is the reaminder, which has to be spread as equally as possible among the steps
			if (remainder<0)
			{
				//reset, if the last index to be assigned with the present step would exceed the last possible
				step--;
				last_assigned=step*(pnused_atoms[itype]-1);//this is the last particle index to be assigned with step starting with 0
				last_to_assign=pnatoms[itype]-1;//this is the index of the last particle that should be assigned to this type
				remainder=last_to_assign-last_assigned;//this is the reaminder, which has to be spread as equally as possi
			}
			
			//Setting the first and the last particle for this type 
			*ind_finder[itype]=cumul[itype];
			*(ind_finder[itype]+pnused_atoms[itype]-1)=cumul[itype+1]-1;
		}
		else
		{
			step=0;//if no step could be calculated, as only one particle could be assigned to this type
			*ind_finder[itype]=cumul[itype];
		}
	

		//setting inc_step, the step value will be increased with one in every inc_step-th assignment which was done with the original step value
		if (remainder>0)
			inc_step=(int)pnused_atoms[itype]/remainder;
		else
			inc_step=pnatoms[itype];//no increasing necessary, setting it to pnatoms[itype] which cannot occure 
		count=1;//counter for consecutive steps with original step size, to help to spread the remainder equally
		n_assigned=2;//counting the number of assigned particles
		last_assigned=0;//the index of the first tdiff value, that has been assigned already
		for (n=0;n<pnused_atoms[itype]-2;n++)//first and last has been assigned
		{
			//setting the step value 
			if (count==inc_step && remainder>0)
			{
				present_step=step+1;//it is time to decrease the remainder by incresing the step size
				remainder--;//decrease the remainder
			}
			else
				present_step=step;//use the original step size

			//index of the atom in the configuration (between 0->nparticles-1) to be selected for output
			*(ind_finder[itype]+1+n)=cumul[itype]+last_assigned+present_step;
			n_assigned++;
			last_assigned+=present_step;
			if (present_step==step)
				count++;//this value was assigned with the original step size, increase the count
		}//end of n cycle 

		if (n_assigned!=pnused_atoms[itype])
		{	
			cout<<"Only "<<n_assigned<<" particles from the intended "<<pnused_atoms[itype]<<" is selected"<<endl;
			pnused_atoms[itype]=n_assigned;
		}


		//ordering the particle indices for this type
		O_Sort(pnused_atoms[itype],ind_finder[itype]);
		for (n=0;n<pnused_atoms[itype];n++)
			outfile<<*(ind_finder[itype]+n)<<endl;

	}//end of itype cycle	

outfile.close();
}

//Calculating the indices of the particles positinoned between the two cutting planes
void SimpleCfg::SelectIndex()
{
	int itype, i,tot;
	int *ind;
	double *pos;
	indices=new int[ntotal];

	//choose_part==4, choose the particles between the cutting planes
	tot=0;

	ind=indices;
	ind_finder[0]=indices;
	for (itype=0; itype<ntypes; itype++)
	{   
		pnused_atoms[itype]=0;//counting the atoms between coord_limit1 and coord_limit2 for the given type
		pos=finder[itype];//sets the pointer to the first coordinate of this type
		
		for (i=cumul[itype]; i<cumul[itype+1]; i++)//go through all the atoms of each type  
		{
			//decide, whteher this atom is between the limits
			if (*(pos+select_axis)>=coord_limit1 && *(pos+select_axis)<=coord_limit2)
			{
				pnused_atoms[itype]++;
				*ind++=i;//add the index to the indices
			}

			pos+=3;//goto next atom

		}
		if (pnused_atoms[itype]>0)
		{
			ind_finder[itype]=indices+tot;
			tot+=pnused_atoms[itype];
		}
			
	}
};

//In case of choose_part=3, check, whether the indices are in range
void SimpleCfg::CheckIndex()
{
	int i,j,itype;

	i=0;
	for (itype=0;itype<ntypes;itype++)
	{
		for (j=0;j<pnused_atoms[itype];j++)
		{
			if (indices[i]<cumul[itype] || indices[i]>=cumul[itype+1])
			{
				cout<<"****WARNING****"<<endl;
				cout<<"The atom index "<<indices[i]+1<<" read from the parameter file for CrystalMaker for a "<<itype;
				cout<<". type atom is out of range for this type. The range is from "<<cumul[itype]<<" to "<<cumul[itype+1]-1<<"!"<<endl;
				cout<<"Check the configuration file! Cannot run this way, exiting..."<<endl;
				cleanexit();
			}
			i++;
		}
	}
}