//convert_CMconfigs.cpp

//Converting RMC++ configurations to CrystalMaker
//Written by Orsolya Gereben
//Last changed 06.11.2010

#include "Coordinates.h" 

int main(int argc, char* argv[])
{
	int i,itime;
	int ext_digit=4;//number of digits the time frame index is represented in the outfile name
	int nleading_zero;//number of leading zeros before the time frame index in the outfile name
	char key;
	char out_cfg[67], in_cfg[65], out_cfg_frame[75],parfilename[60];
	char *frame_ext;//character representation of the time frame index
	ifstream infile;
	ofstream outfile,scrfile;

	cout<<"This program creates CrytalMaker *.cmtx text format configuration file from "<<endl;
	cout<<"version 3 format  RMC++ file.\n"<<endl;
	cout<<"It requires a parameter file containing information for the building of the "<<endl;
	cout<<"CrystalMaker configuration file."<<endl;
	cout<<"1. line: has to contain the full name of the configuration file. It can be a "<<endl;
	cout<<"       -text type *.cfg file containing one configuration "<<endl;
	cout<<"       -text type *_coll.cfg file containing consecutively several "<<endl;
	cout<<"        configurations"<<endl;
	cout<<"       -binary *.bcf file conaining one configuration."<<endl;
	cout<<"     The name of the output file will be derived from the input file name, "<<endl;
	cout<<"     cutting the extension, inserting the frame index in case of multiple "<<endl;
	cout<<"     configurations, and adding the cmtx extension."<<endl;
	cout<<"2. line: number of configurations to convert, always 1 except for *_coll.cfg"<<endl;
	cout<<"         file."<<endl;
	cout<<"3. line: the number of atom types in the configuration"<<endl;
	cout<<"4. line: integer option switch regarding how the particles should"<<endl;
	cout<<"         be selected, if not all the particles of a type will be used."<<endl;
	cout<<"         If the option switch=4, than one more integer "<<endl;
	cout<<"         and two real numbers have to follow in the same line, see later.\n"<<endl;
	cout<<"   option 1: The indices of the specified number of atoms for each type will"<<endl;
	cout<<"             be spread among the atoms of this type with as equal step as "<<endl;
	cout<<"             possible. As usually atoms with similar indices are close to "<<endl;
	cout<<"             each other, especially if no swaps were used during the "<<endl;
	cout<<"             simulation, this way the particles can be found everywhere in "<<endl;
	cout<<"             the simulation box."<<endl;
	cout<<"   option 2: The first given number of atoms will be used from a type. "<<endl;
	cout<<"             (Most probably atoms close to each other are chosen this way!)"<<endl;
	cout<<"   option 3: Atoms of given indices specified at the end the parameter file"<<endl;
	cout<<"             will be selected. (Index can range from 1 -> ntotal, has to  "<<endl;
	cout<<"             follow the order of atom types.)"<<endl;
	cout<<"   option 4: Only particles between two parallel planes will be saved into the"<<endl;
	cout<<"             file. The planes have to be parallel to one of the sides of the "<<endl;
	cout<<"             simulation box! There have to be an integer after option 4 to "<<endl;
	cout<<"             specify which axis is perpendicular to the plane (x:0, y:1, z:2),"<<endl;
	cout<<"             followed by two real numbers (between -1 and +1) to specify the "<<endl;
	cout<<"             reduced coordinates,between which the atoms will be outputted."<<endl;
	cout<<"             Example for this line of the parameter file choosing the atoms,"<<endl;
	cout<<"             with reduced y-coordinates between -0.7 -> 0.5:   4  1  -0.7 0.5  "<<endl;

	cout<<"\nAfter this it has to be as many lines as atom types, in each line containing:"<<endl;
	cout<<"its chemical SYMBOL, the NUMBER OF PARTICLES used from this type."<<endl;
	cout<<"\nIn case of option 3 there has to follow the indices of the atoms to select one"<<endl;
	cout<<"index in each line. Indexing starts with 1, first the indices for the first "<<endl;
	cout<<"atom type, then the second and so on."<<endl;
	
	//For a multicomponent system, if a component has to be left out from the CrystalMaker configuration than give 0 for the 
	//number of used particles!

	//========getting information phase=========================
	if (argc==1)//no command line arguments were specified
	{
		cout<<"\nGive the name of the parameter file containing the information "<<endl;
		cout<<"for the conversion:"<<endl;
		cin>>parfilename;
	}
	else //there were command line arguments, the first is assumed to be the file name
		strcpy(parfilename,argv[1]);


	
	//reading the parameters for building the CrystalMaker configuration file, just in case of the first time frame
	SimpleCfg::GetCrystalMakerPar(parfilename, in_cfg, out_cfg);

	if (SimpleCfg::option & 1)
	{
		cleanopen(infile,in_cfg);
		SimpleCfg::GetParams_cfg(infile);
		infile.close();
	}
	else
	{
		cleanopen_bin(infile,in_cfg);
		SimpleCfg::GetParams_bcf(infile);
		infile.close();
	}


	SimpleCfg config;
	if (SimpleCfg::option & 1)	
		cleanopen(infile,in_cfg);
	else
		cleanopen_bin(infile,in_cfg);
			
	//get the parameters
	for (itime=0; itime<SimpleCfg::nconfig;itime++)
	{
				
		//Creating the file names for the output file in case of more than 1 configuration in the format outname_000X.cmtx where X is the frame index
		//This format is appropriate for the CrystalMaker
		strcpy(out_cfg_frame,out_cfg);
		if (SimpleCfg::nconfig>1)
		{
			frame_ext=IntToStr(itime);//creating the extension for the files of this time frame
			nleading_zero=ext_digit-int(log10(itime))-1;//number of zeros before the frame index
			for (i=0;i<nleading_zero;i++)
				strcat(out_cfg_frame,"0");//adding the leading zeros
			strcat(out_cfg_frame,frame_ext);
		}
		strcat(out_cfg_frame,".cmtx");
			
		cout<<"\nConverting the "<<itime+1<<" configuration of the "<<in_cfg<<" file to ";
		cout<<"CrystalMaker format configuration file "<<out_cfg_frame<<endl;	
	
		//Load the RMC configuration
		if (SimpleCfg::option & 1)
			config.Load(infile);
		else
			config.Load_binary(infile);
		

		if (SimpleCfg::choose_part==1 && itime==0)
			config.AssignIndex();
		
		if (SimpleCfg::choose_part==3)
			config.CheckIndex();//checking the indices read from the parameter file

		if (SimpleCfg::choose_part==4)
			config.SelectIndex();//selecting the atoms among the cutting planes

		//Saving the CrystalMaker Extended format text file
		config.SaveCrystalMaker(out_cfg_frame);//the file is opened and closed by this routine
		
		if (itime<SimpleCfg::nconfig-1)
		{
			skipline(infile);
			skipline(infile);
		}

		
					
	}
	infile.close();		
	
	cout<<" Finished execution, quit (y)? ->"<<endl;;
	cin>>key;
	return(0);
};
